@@ -13,15 +13,37 @@ from . classes import (weightmap,aggregated)
 def process_weights(ds,weights=None,target='ds'):
     """ Process weights - including regridding
     
-    If target == 'ds', regrid weights to ds. If target == 'weights',
-    regrid ds to weights. 
+    If ``target == 'ds'``, regrid `weights` to `ds`. If ``target == 'weights'``,
+    regrid `ds` to `weights`. 
+
+    Parameters
+    ---------------
+    ds : :class:`xarray.Dataset`, :class:`xarray.DataArray`
+        an :class:`xarray.Dataset`/:class:`xarray.DataArray` to regrid
     
-    Also needs an output that will go into gdf_out, with a flag for
-    'something was regridded, y'all'
-    
-    ohhh... wait what if the pixel polygons have the weight in the 
-    geodataframe... so this actually goes in the get_pixel_polygons
+    weights : :class:`xarray.DataArray`, optional, default = ``None``
+        an :class:`xarray.DataArray` containing a weight (numeric) 
+        at each location
     
+    target : str, optional, default = ``'ds'``
+        whether weights should be regridded to the `ds` grid (by
+        default) or vice-versa (not yet supported, returns 
+        `NotImplementedError`)
+
+
+    Returns
+    ---------------
+    ds : :class:`xarray.Dataset`, :class:`xarray.DataArrays`
+        the input :class:`xarray.Dataset`/:class:`xarray.DataArray`, with a new variable 
+        `weights` specifying weights for each pixel
+
+    weights_info : dict 
+        a dictionary storing information about the 
+        weights regridding process, with the fields:
+
+        - ``target``: showing which of the two grids was retained
+        - ``ds_grid``: a dictionary with the grid ``{"lat":ds.lat,"lon",ds.lon}``
+        - ``weights_grid``: a dictionary with the grid ``{"lat":weights.lat,"lon":weights.lon}``
     """
     
     if weights is None:
@@ -78,41 +100,10 @@ def create_raster_polygons(ds,
                            mask=None,subset_bbox=None,
                            weights=None,weights_target='ds'):
     """ Create polygons for each pixel in a raster
-    
-    Keyword arguments:
-    ds -- an xarray dataset with the variables 
-          'lat_bnds' and 'lon_bnds', which are both
-          lat/lon x 2 arrays giving the min and 
-          max values of lat and lon for each pixel
-          given by lat/lon
-    subset_bbox -- by default None; if a geopandas
-                   geodataframe is entered, the bounding
-                   box around the geometries in the gdf 
-                   are used to mask the grid, to reduce
-                   the number of pixel polygons created
-    mask -- ## THIS IS WHERE MASKS CAN BE ADDED - 
-    # I.E. AN OCEAN MASK. OR MAYBE EVEN ALLOW 
-    # SHAPEFILES TO BE ADDED AND CALCULATED
-    # THE MASKED PIXELS ARE JUST IGNORED, AND NOT 
-    # ADDED. will make identifying them harder
-    # in the first bit of aggregate, but could 
-    # make processing faster if you have a ton
-    # of ocean pixels or something... 
-          
-    Returns:
-    a dictionary containing:
-    - "gdf_pixels": a geopandas geodataframe containing a 
-                    'geometry' giving the pixel boundaries 
-                    for each 'lat' / 'lon' pair
-    - "source_grid": a dictionary containing the original
-                     lat and lon inputs under the keys
-                     "lat" and "lon" (just the 
-                     xr.DataArrays of those variables in
-                     the input ds)
-                  
+
     Note: 
     'lat_bnds' and 'lon_bnds' can be created through the
-    'get_bnds' function if they are not already included
+    :func:`xagg.aux.get_bnds` function if they are not already included
     in the input raster file. 
     
     Note:
@@ -120,6 +111,38 @@ def create_raster_polygons(ds,
     rectangular grids (so where every pixel side is
     a straight line in lat/lon space). Future versions
     may include support for irregular grids. 
+    
+    Parameters
+    ---------------
+    ds : :class:`xarray.Dataset`
+        an xarray dataset with the variables 
+        'lat_bnds' and 'lon_bnds', which are both
+        lat/lon x 2 arrays giving the min and 
+        max values of lat and lon for each pixel
+        given by lat/lon
+    subset_bbox : :class:`geopandas.GeoDataFrame`, optional, default = ``None`` 
+        if a :class:`geopandas.GeoDataFrame`
+        is entered, the bounding
+        box around the geometries in the `gdf` 
+        are used to mask the grid, to reduce
+        the number of pixel polygons created
+          
+    Returns
+    ---------------
+    pix_agg: dict
+        a dictionary containing:
+
+        - ``'gdf_pixels'`` 
+            a :class:`geopandas.GeoDataFrame` containing a 
+            'geometry' giving the pixel boundaries 
+            for each 'lat' / 'lon' pair
+        - ``'source_grid'`` 
+            a dictionary containing the original
+            lat and lon inputs under the keys
+            "lat" and "lon" (just the 
+            :class:`xarray.DataArray` of those variables in
+            the input `ds`)
+                      
     """
     
     # Standardize inputs
@@ -205,47 +228,64 @@ def create_raster_polygons(ds,
 def get_pixel_overlaps(gdf_in,pix_agg):
     """ Get, for each polygon, the pixels that overlap and their area of overlap
     
-    Finds, for each polygon in gdf_in, which pixels intersect it, and by how much. 
+    Finds, for each polygon in `gdf_in`, which pixels intersect it, and by how much. 
     
     Note: 
     Uses EASE-Grid 2.0 on the WGS84 datum to calculate relative areas
     (see https://nsidc.org/data/ease)
     
-    Keyword arguments:
-    gdf_in     -- a GeoPandas GeoDataFrame giving the polygons over which 
-                  the variables should be aggregated. Can be just a read
-                  shapefile (with the added column of "poly_idx", which 
-                  is just the index as a column).
-    pix_agg    -- the output of [create_raster_polygons]; a dict containing:
-                    'gdf_pixels': a GeoPandas GeoDataFrame giving for each row 
-                                  the columns "lat" and "lon" (with coordinates) 
-                                  and a polygon giving the boundary of the pixel 
-                                  given by lat/lon
-                    'source_grid':[da.lat,da.lon] of the grid used to create
-                                   the pixel polygons
-                  
-    Returns:
-    A dictionary containing: 
-    ['agg']: a dataframe containing all the fields of [gdf_in] (except
-             geometry) and the additional columns 
-                coords:   the lat/lon coordiates of all pixels that overlap
-                          the polygon of that row
-                pix_idxs: the linear indices of those pixels within the 
-                          gdf_pixels grid
-                rel_area: the relative area of each of the overlaps between
-                          the pixels and the polygon (summing to 1 - e.g. 
-                          if the polygon is exactly the size and location of
-                          two pixels, their rel_areas would be 0.5 each)
-    ['source_grid']: a dictionary with keys 'lat' and 'lon' giving the 
-                     original lat/lon grid whose overlaps with the polygons
-                     was calculated
-    ['geometry']: just the polygons from [gdf_in]                                        
+    Parameters
+    ---------------
+    gdf_in : :class:`geopandas.GeoDataFrane`
+        a :class:`geopandas.GeoDataFrame` giving the polygons over which 
+        the variables should be aggregated. Can be just a read
+        shapefile (with the added column of "poly_idx", which 
+        is just the index as a column).
+    pix_agg : dict
+        the output of :func:`xagg.core.create_raster_polygons`; a dict containing:
+
+        - ``'gdf_pixels'``
+            a :class:`geopandas.GeoDataFrame` giving for each row 
+            the columns "lat" and "lon" (with coordinates) 
+            and a polygon giving the boundary of the pixel 
+            given by lat/lon
+        - ``'source_grid'``
+            ``[da.lat,da.lon]`` of the grid used to create
+            the pixel polygons
+    
+
+    Returns
+    ---------------
+    wm_out: dict
+        A dictionary containing: 
+        
+        - ``'agg'``
+            a dataframe containing all the fields of `gdf_in` (except
+            geometry) and the additional columns: 
+
+            - `coords`:  the lat/lon coordiates of all pixels that overlap
+             the polygon of that row
+            - `pix_idxs`: the linear indices of those pixels within the 
+             `gdf_pixels` grid
+            - `rel_area`: the relative area of each of the overlaps between
+             the pixels and the polygon (summing to 1 - e.g. 
+             if the polygon is exactly the size and location of
+             two pixels, their rel_areas would be 0.5 each)
+        - ``'source_grid'``: 
+            a dictionary with keys 'lat' and 'lon' giving the 
+            original lat/lon grid whose overlaps with the polygons
+            was calculated
+        - ``'geometry'``: 
+            just the polygons from `gdf_in` 
+
+
     """
     
     
     # Add an index for each polygon as a column to make indexing easier
-    if 'poly_idx' not in gdf_in.columns:
-        gdf_in['poly_idx'] = gdf_in.index.values
+    #if 'poly_idx' not in gdf_in.columns:
+    #    gdf_in['poly_idx'] = gdf_in.index.values
+    gdf_in['poly_idx'] = np.arange(0,len(gdf_in))
         
     # Match up CRSes
     pix_agg['gdf_pixels'] = pix_agg['gdf_pixels'].to_crs(gdf_in.crs)
@@ -311,42 +351,60 @@ def get_pixel_overlaps(gdf_in,pix_agg):
 def aggregate(ds,wm):
     """ Aggregate raster variable(s) to polygon(s)
     
-    Aggregates (N-D) raster variables in [ds] to the polygons
-    in [gfd_out] - in other words, gives the weighted average
+    Aggregates (N-D) raster variables in `ds` to the polygons
+    in `gfd_out` - in other words, gives the weighted average
     of the values in [ds] based on each pixel's relative area
     overlap with the polygons. 
     
-    The values will be additionally weighted if a [weight] was
-    inputted into [create_raster_polygons]
+    The values will be additionally weighted if a `weight` was
+    inputted into :func:`xagg.core.create_raster_polygons`
     
-    The code checks whether the input lat/lon grid in [ds] is 
-    equivalent to the linearly indexed grid in [wm], or 
+    The code checks whether the input lat/lon grid in `ds` is 
+    equivalent to the linearly indexed grid in `wm`, or 
     if it can be cropped to that grid. 
     
     
-    Keyword arguments:
-    ds      -- an xarray dataset containing one or more
-               variables with dimensions lat, lon (and possibly
-               more). The dataset's geographic grid has to 
-               include the lat/lon coordinates used in 
-               determining the pixel overlaps in 
-               [get_pixel_overlaps] (and saved in 
-               wm['source_grid'])
+    Parameters
+    ---------------
+    ds : :class:`xarray.Dataset`
+        an :class:`xarray.Dataset` containing one or more
+        variables with dimensions lat, lon (and possibly
+        more). The dataset's geographic grid has to 
+        include the lat/lon coordinates used in 
+        determining the pixel overlaps in 
+        :func:`xagg.core.get_pixel_overlaps` (and saved in 
+        ``wm['source_grid']``)
                
-    wm 		-- the output to [get_pixel_overlaps]; a 
-               [weightmap] object containing ['agg'] - a dataframe, 
-               with one row per polygon, and the columns pix_idxs 
-               and rel_area, giving the linear indices and 
-               the relative area of each pixel over the polygon,
-               respectively; and ['source_grid'] describing 
-               the lat/lon grid on which the aggregating parameters
-               were calculated (and on which the linear indices 
-               are based)
+    wm : :class:`xagg.classes.weightmap`
+        the output to :func:`xagg.core.get_pixel_overlaps`; a 
+        :class:`xagg.classes.weightmap` object containing 
+
+        - ``['agg']``
+            a dataframe, 
+            with one row per polygon, and the columns `pix_idxs`
+            and `rel_area`, giving the linear indices and 
+            the relative area of each pixel over the polygon,
+            respectively 
+        - ``['source_grid']``  
+            the lat/lon grid on which the aggregating parameters
+            were calculated (and on which the linear indices 
+            are based)
                
-    Returns:
-    An xa.aggregated object with the aggregated variables 
+    Returns
+    ---------------
+    agg_out : :class:`xagg.classes.aggregated`
+        an :class:`xagg.classes.aggregated` object with the aggregated variables 
     
     """
+    # Turn into dataset if dataarray
+    if type(ds)==xr.core.dataarray.DataArray:
+      if ds.name is None:
+        warnings.warn('An unnamed xr.DataArray was inputted instead of a xr.Dataset; the output variable will be "var"')
+        ds = ds.to_dataset(name='var')
+      else:
+        ds = ds.to_dataset()
+
+
     # Run ds through fix_ds (to fix lat/lon names, lon coords)
     ds = fix_ds(ds)
 
@@ -402,7 +460,7 @@ def aggregate(ds,wm):
 
                     if not np.isnan(ds[var].isel(loc=wm.agg.iloc[poly_idx,:].pix_idxs)).all(): 
                         # Get relative areas for the pixels overlapping with this Polygon
-                        tmp_areas = np.squeeze(wm.agg.iloc[poly_idx,:].rel_area)
+                        tmp_areas = np.atleast_1d(np.squeeze(wm.agg.iloc[poly_idx,:].rel_area))
                         # Replace overlapping pixel areas with nans if the corresponding pixel
                         # is only composed of nans
                         tmp_areas[np.array(np.isnan(ds[var].isel(loc=wm.agg.iloc[poly_idx,:].pix_idxs)).all(other_dims).values)] = np.nan
